@@ -9,12 +9,14 @@ import argparse
 import logging
 import sys
 
-from .cli_utils import (examine,
-                        load_blacklist, load_input_urls,
+from concurrent.futures import ThreadPoolExecutor, as_completed
+
+from .cli_utils import (load_blacklist, load_input_dict, load_input_urls,
+                        convert_inputlist,
                         file_processing_pipeline, url_processing_pipeline,
-                        write_result)
+                        examine, write_result)
 from .feeds import find_feed_urls
-from .settings import SLEEP_TIME
+from .settings import DOWNLOAD_THREADS, SLEEP_TIME
 from .sitemaps import sitemap_search
 
 
@@ -64,7 +66,7 @@ def parse_args(args):
                         help="look for feeds and/or pass a feed URL as input",
                         nargs='?', const=True, default=False)
     group1.add_argument("--sitemap",
-                        help="look for sitemaps URLs for the given website",
+                        help="look for sitemaps for the given website and/or enter a sitemap URL",
                         nargs='?', const=True, default=False)
     group1.add_argument("--list",
                         help="return a list of URLs without downloading them",
@@ -73,17 +75,27 @@ def parse_args(args):
                         help="""name of file containing already processed or
                                 unwanted URLs to discard during batch processing""",
                         type=str)
+    group1.add_argument('--url-filter',
+                        help="only process/output URLs containing these patterns (space-separated strings)",
+                        nargs='+', type=str)
     group1.add_argument('--backup-dir',
-                        help="Preserve a copy of downloaded files in a backup directory",
+                        help="preserve a copy of downloaded files in a backup directory",
                         type=str)
     group1.add_argument('--timeout',
-                        help="Use timeout for file conversion to prevent bugs",
+                        help="use timeout for file conversion to prevent bugs",
                         action="store_true")
     group1.add_argument('--parallel',
-                        help="Specify a number of cores/threads for parallel downloads and/or processing",
+                        help="specify a number of cores/threads for parallel downloads and/or processing",
                         type=int)
     group1.add_argument('--keep-dirs',
-                        help="Keep input directory structure and file names",
+                        help="keep input directory structure and file names",
+                        action="store_true")
+    group1.add_argument('--hash-as-name',
+                        help="""use file content hash as output file name (for deduplication)
+                        instead of random default""",
+                        action="store_true")
+    group1.add_argument('--archived',
+                        help='try to fetch URLs from the Internet Archive if downloads fail',
                         action="store_true")
 
     # https://docs.python.org/3/library/argparse.html#argparse.ArgumentParser.add_mutually_exclusive_group
@@ -126,8 +138,11 @@ def parse_args(args):
                         help="select a target language (ISO 639-1 codes)",
                         type=str)
     group3.add_argument("--deduplicate",
-                        help="Filter out duplicate documents and sections",
+                        help="filter out duplicate documents and sections",
                         action="store_true")
+    group3.add_argument("--config-file",
+                        help="override standard extraction parameters with a custom config file",
+                        type=str)
     return parser.parse_args()
 
 
@@ -162,26 +177,37 @@ def process_args(args):
     # processing according to mutually exclusive options
     # read url list from input file
     if args.inputfile and args.feed is False and args.sitemap is False:
-        input_urls = load_input_urls(args.inputfile)
-        url_processing_pipeline(args, input_urls, SLEEP_TIME)
-    # fetch urls from a feed
-    elif args.feed:
+        inputdict = load_input_dict(args.inputfile, args.blacklist)
+        url_processing_pipeline(args, inputdict, SLEEP_TIME)
+    # fetch urls from a feed or a sitemap
+    elif args.feed or args.sitemap:
+        inputdict = None
+        download_threads = args.parallel or DOWNLOAD_THREADS
+        # load input URLs
         if args.inputfile:
             input_urls = load_input_urls(args.inputfile)
-        else:
+        elif args.feed:
             input_urls = [args.feed]
-        for url in input_urls:
-            links = find_feed_urls(url)
-            url_processing_pipeline(args, links, SLEEP_TIME)
-    # fetch urls from a sitemap
-    elif args.sitemap:
-        if args.inputfile:
-            input_urls = load_input_urls(args.inputfile)
-        else:
+        elif args.sitemap:
             input_urls = [args.sitemap]
-        for url in input_urls:
-            links = sitemap_search(url, target_lang=args.target_language)
-            url_processing_pipeline(args, links, SLEEP_TIME)
+        # link discovery and storage
+        if args.feed:
+            with ThreadPoolExecutor(max_workers=download_threads) as executor:
+                future_to_url = {executor.submit(find_feed_urls, url): url for url in input_urls}
+                for future in as_completed(future_to_url):
+                    if future.result() is not None:
+                        inputdict = convert_inputlist(args.blacklist, future.result(), args.url_filter, inputdict)
+        elif args.sitemap:
+            with ThreadPoolExecutor(max_workers=download_threads) as executor:
+                future_to_url = {executor.submit(sitemap_search, url, target_lang=args.target_language): url for url in input_urls}
+                for future in as_completed(future_to_url):
+                    if future.result() is not None:
+                        inputdict = convert_inputlist(args.blacklist, future.result(), args.url_filter, inputdict)
+        # add simple conversion
+        else:
+            inputdict = convert_inputlist(args.blacklist, input_urls, args.url_filter)
+        # processing
+        url_processing_pipeline(args, inputdict, SLEEP_TIME)
     # read files from an input directory
     elif args.inputdir:
         file_processing_pipeline(args)
@@ -189,9 +215,8 @@ def process_args(args):
     else:
         # process input URL
         if args.URL:
-            url_processing_pipeline(args, [args.URL], 0)  # process single url
-            #if htmlstring is None:
-            #    sys.exit('ERROR: no valid result for url: ' + args.URL + '\n')
+            inputdict = convert_inputlist(args, [args.URL], None)
+            url_processing_pipeline(args, inputdict, 0)  # process single url
         # process input on STDIN
         else:
             # file type and unicode check
