@@ -11,19 +11,21 @@ Module bundling all functions needed to extract the text in a webpage.
 import logging
 import re # import regex as re
 
+from collections import OrderedDict
 from copy import deepcopy
 
 from lxml import etree, html
 
 # own
 from .external import justext_rescue, sanitize_tree, SANITIZED_XPATH, try_readability
-from .filters import content_fingerprint, duplicate_test, language_filter, text_chars_test
+from .filters import (check_html_lang, content_fingerprint, duplicate_test,
+                     language_filter, text_chars_test)
 from .htmlprocessing import (convert_tags, discard_unwanted,
                              discard_unwanted_comments, handle_textnode,
-                             link_density_test, link_density_test_tables, process_node, tree_cleaning)
+                             link_density_test, link_density_test_tables,
+                             process_node, tree_cleaning)
 from .metadata import extract_metadata, METADATA_LIST
-from .settings import (MIN_EXTRACTED_SIZE, MIN_EXTRACTED_COMM_SIZE,
-                       MIN_OUTPUT_SIZE, MIN_OUTPUT_COMM_SIZE, TAG_CATALOG)
+from .settings import use_config, DEFAULT_CONFIG, TAG_CATALOG
 from .utils import load_html, trim, txttocsv, is_image_file
 from .xml import (build_json_output, build_xml_output, build_tei_output,
                   control_xml_output, xmltotxt)
@@ -59,7 +61,7 @@ def handle_formatting(element):
     return processed_element
 
 
-def handle_lists(element, dedupbool):
+def handle_lists(element, dedupbool, config):
     '''Process lists elements'''
     processed_element = etree.Element(element.tag)
     for child in element.iter('item'):
@@ -72,7 +74,7 @@ def handle_lists(element, dedupbool):
         else:
             # proceed with iteration, fix for nested elements
             for subelem in child.iter():
-                processed_subchild = handle_textnode(subelem, comments_fix=False, deduplicate=dedupbool)
+                processed_subchild = handle_textnode(subelem, comments_fix=False, deduplicate=dedupbool, config=config)
                 # add child element to processed_element
                 if processed_subchild is not None:
                     subchildelem = etree.SubElement(newchildelem, processed_subchild.tag)
@@ -109,14 +111,14 @@ def handle_quotes(element):
     return None
 
 
-def handle_other_elements(element, potential_tags, dedupbool):
+def handle_other_elements(element, potential_tags, dedupbool, config):
     '''Handle diverse or unknown elements in the scope of relevant tags'''
     # delete unwanted
     if element.tag not in potential_tags:
         # LOGGER.debug('discarding: %s %s', element.tag, element.text)
         return None
     if element.tag == 'div':
-        processed_element = handle_textnode(element, comments_fix=False, deduplicate=dedupbool)
+        processed_element = handle_textnode(element, comments_fix=False, deduplicate=dedupbool, config=config)
         if processed_element is not None:
             processed_element.attrib.clear()
             # small div-correction # could be moved elsewhere
@@ -125,11 +127,11 @@ def handle_other_elements(element, potential_tags, dedupbool):
             # insert
             return processed_element
     else:
-        LOGGER.debug('processing other element: %s %s', element.tag, element.text)
+        LOGGER.warning('unexpected element seen: %s %s', element.tag, element.text)
     return None
 
 
-def handle_paragraphs(element, potential_tags, dedupbool):
+def handle_paragraphs(element, potential_tags, dedupbool, config):
     '''Process paragraphs (p) elements along with their children,
        trim and clean the content'''
     element.attrib.clear()
@@ -144,13 +146,13 @@ def handle_paragraphs(element, potential_tags, dedupbool):
     processed_element = etree.Element(element.tag)
     for child in element.iter():
         if child.tag not in potential_tags:
-            LOGGER.debug('unexpected elem in paragraph: %s %s %s', child.tag, child.text, child.tail)
+            LOGGER.warning('unexpected in p: %s %s %s', child.tag, child.text, child.tail)
             continue
-        processed_child = handle_textnode(child, comments_fix=False, deduplicate=dedupbool)
+        processed_child = handle_textnode(child, comments_fix=False, deduplicate=dedupbool, config=config)
         if processed_child is not None:
             # needing attention!
             if child.tag == 'p':
-                LOGGER.debug('extra elem within p: %s %s %s', child.tag, child.text, child.tail)
+                LOGGER.debug('extra p within p: %s %s %s', child.tag, child.text, child.tail)
                 if processed_element.text:
                     processed_element.text += ' ' + trim(child.text)
                 else:
@@ -254,30 +256,30 @@ def handle_image(element):
     return processed_element
 
 
-def recover_wild_paragraphs(tree, result_body, potential_tags=TAG_CATALOG, deduplicate=True):
+def recover_wild_paragraphs(tree, result_body, potential_tags=TAG_CATALOG, deduplicate=True, config=None):
     '''Look for all p-elements, including outside of the determined frame
        and throughout the document to recover potentially missing text parts'''
     LOGGER.debug('Taking all p-elements')
     # prune
     search_tree = discard_unwanted(tree)
     etree.strip_tags(search_tree, 'a', 'link', 'span')
-    processed_elems = [handle_paragraphs(element, potential_tags, deduplicate) for element in search_tree.iter('blockquote', 'code', 'p', 'pre', 'q', 'quote')] # 'head', 'list'
+    processed_elems = [handle_paragraphs(element, potential_tags, deduplicate, config) for element in search_tree.iter('blockquote', 'code', 'p', 'pre', 'q', 'quote')] # 'head', 'list'
     result_body.extend(list(filter(None.__ne__, processed_elems)))
     return result_body
 
 
-def handle_textelem(element, potential_tags, dedupbool):
+def handle_textelem(element, potential_tags, dedupbool, config):
     '''Process text element and determine how to deal with its content'''
     new_element = None
     # bypass: nested elements
     if element.tag == 'list':
-        new_element = handle_lists(element, dedupbool)
+        new_element = handle_lists(element, dedupbool, config)
     elif element.tag == 'quote':   # + 'code'?
         new_element = handle_quotes(element)
     elif element.tag == 'head':
         new_element = handle_titles(element)
     elif element.tag == 'p':
-        new_element = handle_paragraphs(element, potential_tags, dedupbool)
+        new_element = handle_paragraphs(element, potential_tags, dedupbool, config)
     elif element.tag == 'lb':
         if text_chars_test(element.tail) is True:
             element = process_node(element)
@@ -292,11 +294,41 @@ def handle_textelem(element, potential_tags, dedupbool):
         new_element = handle_image(element)
     else:
         # other elements (div, ??, ??)
-        new_element = handle_other_elements(element, potential_tags, dedupbool)
+        new_element = handle_other_elements(element, potential_tags, dedupbool, config)
     return new_element
 
 
-def extract_content(tree, include_tables=False, deduplicate=True):
+def delete_by_link_density(subtree, tagname, backtracking=False):
+    '''Determine the link density of elements with respect to their length,
+       and remove the elements identified as boilerplate.'''
+    myelems, deletions = dict(), list()
+    for elem in subtree.iter(tagname):
+        result, templist = link_density_test(elem)
+        if result is True:
+            deletions.append(elem)
+        elif backtracking is True and len(templist) > 0:
+            text = trim(elem.text_content())
+            if text not in myelems:
+                myelems[text] = [elem]
+            else:
+                myelems[text].append(elem)
+    # summing up
+    if backtracking is True:
+        for item in myelems:
+            if 0 < len(item) < 100 and len(myelems[item]) >= 3:
+                deletions.extend(myelems[item])
+                #print('backtrack:', item)
+            #elif 20 < len(item) < 100 and len(myelems[item]) >= 3:
+            #    deletions.extend(myelems[item])
+            #    print('backtrack 2:', item)
+            #else: # and not re.search(r'[?!.]', text):
+                #print(elem.tag, templist)
+    for elem in list(OrderedDict.fromkeys(deletions)):
+        elem.getparent().remove(elem)
+    return subtree
+
+
+def extract_content(tree, include_tables=False, deduplicate=False, config=None):
     '''Find the main content of a page using a set of XPath expressions,
        then extract relevant elements, strip them of unwanted subparts and
        convert them'''
@@ -312,12 +344,9 @@ def extract_content(tree, include_tables=False, deduplicate=True):
         # prune
         subtree = discard_unwanted(subtree)
         # remove elements by link density
-        for elem in subtree.iter('list'):
-            if link_density_test(elem) is True:
-                elem.getparent().remove(elem)
-        for elem in subtree.iter('div'):
-            if link_density_test(elem) is True:
-                elem.getparent().remove(elem)
+        subtree = delete_by_link_density(subtree, 'div', backtracking=True)
+        subtree = delete_by_link_density(subtree, 'list', backtracking=False)
+        subtree = delete_by_link_density(subtree, 'p', backtracking=False)
         # define iteration strategy
         potential_tags = set(TAG_CATALOG)  # + 'span'?
         if include_tables is True:
@@ -337,39 +366,34 @@ def extract_content(tree, include_tables=False, deduplicate=True):
         # extract content
         # list(filter(None.__ne__, processed_elems))
         result_body.extend([e for e in
-                            [handle_textelem(e, potential_tags, deduplicate) for e in subtree.xpath('.//*')]
+                            [handle_textelem(e, potential_tags, deduplicate, config) for e in subtree.xpath('.//*')]
                             if e is not None])
         # remove trailing titles
-        try:
-            while result_body[-1].tag == 'head' and result_body[-1].tail is None:
-                # print(lastelem.tag, lastelem.text, lastelem.tail)
-                result_body[-1].getparent().remove(result_body[-1])
-        except IndexError:
-            continue
+        while len(result_body) > 0 and result_body[-1].tag in ('fw', 'head'): # and result_body[-1].tail is None:
+            result_body[-1].getparent().remove(result_body[-1])
         # exit the loop if the result has children
-        if len(result_body) > 0:
-            sure_thing = True
+        if len(result_body) > 1: # try to change this to 0 or 2
             LOGGER.debug(expr)
             break
-    # try parsing wild <p> elements if nothing found or text too short
     temp_text = trim(' '.join(result_body.itertext()))
-    len_text = len(temp_text)
-    if len(result_body) == 0 or len_text < MIN_EXTRACTED_SIZE:
-        result_body = recover_wild_paragraphs(tree, result_body, deduplicate=deduplicate)
+    # try parsing wild <p> elements if nothing found or text too short
+    if len(result_body) == 0 or len(temp_text) < config.getint('DEFAULT', 'MIN_EXTRACTED_SIZE'):
+        result_body = recover_wild_paragraphs(tree, result_body, deduplicate=deduplicate, config=config)
         temp_text = trim(' '.join(result_body.itertext()))
-        len_text = len(temp_text)
+    else:
+        sure_thing = True
     # filter output
     etree.strip_elements(result_body, 'done')
     etree.strip_tags(result_body, 'div')
     # return
-    return result_body, temp_text, len_text, sure_thing
+    return result_body, temp_text, len(temp_text), sure_thing
 
 
-def process_comments_node(elem, potential_tags, dedupbool):
+def process_comments_node(elem, potential_tags, dedupbool, config):
     '''Process comment node and determine how to deal with its content'''
     if elem.tag in potential_tags:
         # print(elem.tag, elem.text_content())
-        processed_element = handle_textnode(elem, comments_fix=True, deduplicate=dedupbool)
+        processed_element = handle_textnode(elem, comments_fix=True, deduplicate=dedupbool, config=config)
         # test length and remove
         if processed_element is not None: # and processed_element.text not in COMMENTS_BLACKLIST:
             processed_element.attrib.clear()
@@ -379,7 +403,7 @@ def process_comments_node(elem, potential_tags, dedupbool):
     return None
 
 
-def extract_comments(tree, dedupbool):
+def extract_comments(tree, dedupbool, config):
     '''Try and extract comments out of potential sections in the HTML'''
     comments_body = etree.Element('body')
     # define iteration strategy
@@ -399,7 +423,7 @@ def extract_comments(tree, dedupbool):
         #    processed_elem = process_comments_node(elem, potential_tags)
         #    if processed_elem is not None:
         #        comments_body.append(processed_elem)
-        processed_elems = [process_comments_node(elem, potential_tags, dedupbool) for elem in subtree.xpath('.//*')]
+        processed_elems = [process_comments_node(elem, potential_tags, dedupbool, config) for elem in subtree.xpath('.//*')]
         comments_body.extend(list(filter(None.__ne__, processed_elems)))
         # control
         if len(comments_body) > 0:  # if it has children
@@ -412,7 +436,7 @@ def extract_comments(tree, dedupbool):
     return comments_body, temp_comments, len(temp_comments), tree
 
 
-def compare_extraction(tree, backup_tree, url, body, text, len_text, target_language, include_formatting):
+def compare_extraction(tree, backup_tree, url, body, text, len_text, target_language, include_formatting, config):
     '''Decide whether to choose own or external extraction
        based on a series of heuristics'''
     # bypass
@@ -433,7 +457,7 @@ def compare_extraction(tree, backup_tree, url, body, text, len_text, target_lang
         algo_flag = False
     elif len_algo > 2*len_text:
         algo_flag = True
-    elif not body.xpath('//p//text()') and len_algo > MIN_EXTRACTED_SIZE:
+    elif not body.xpath('//p//text()') and len_algo > config.getint('DEFAULT', 'MIN_EXTRACTED_SIZE'):
         algo_flag = True  # borderline case
     else:
         LOGGER.debug('extraction values: %s %s for %s', len_text, len_algo, url)
@@ -454,7 +478,7 @@ def compare_extraction(tree, backup_tree, url, body, text, len_text, target_lang
             # post-processing: remove unwanted sections
             body, text, len_text = sanitize_tree(body, include_formatting)
     # try with justext
-    elif len_text < MIN_EXTRACTED_SIZE:
+    elif len_text < config.getint('DEFAULT', 'MIN_EXTRACTED_SIZE'):
         LOGGER.error('not enough text %s', url)
         body, text, len_text, jt_result = justext_rescue(tree, url, target_language, body, len_text, text)
         LOGGER.debug('justext length %s', len_text)
@@ -473,7 +497,17 @@ def compare_extraction(tree, backup_tree, url, body, text, len_text, target_lang
 
 
 def baseline(filecontent):
-    """Use baseline extraction function targeting JSON metadata and/or text paragraphs"""
+    """Use baseline extraction function targeting text paragraphs and/or JSON metadata.
+
+    Args:
+        filecontent: HTML code as binary string or string.
+
+    Returns:
+        A LXML <body> element containing the extracted paragraphs,
+        the main text as string, and its length as integer.
+
+    """
+
     tree = load_html(filecontent)
     postbody = etree.Element('body')
     if tree is None:
@@ -566,15 +600,49 @@ def bare_extraction(filecontent, url=None, no_fallback=False,
                     include_comments=True, output_format='python', target_language=None,
                     include_tables=True, include_images=False, include_formatting=False, deduplicate=False,
                     date_extraction_params=None, with_metadata=False, max_tree_size=None,
-                    url_blacklist=None):
-    '''Internal function for text extraction returning bare Python variables'''
+                    url_blacklist=None, config=DEFAULT_CONFIG):
+    """Internal function for text extraction returning bare Python variables.
+
+    Args:
+        filecontent: HTML code as string.
+        url: URL of the webpage.
+        no_fallback: Skip the backup extraction with readability-lxml and justext.
+        include_comments: Extract comments along with the main text.
+        output_format: Define an output format, Python being the default
+            and the interest of this internal function.
+        target_language: Define a language to discard invalid documents (ISO 639-1 format).
+        include_tables: Take into account information within the HTML <table> element.
+        include_images: Take images into account.
+        include_formatting: Keep structural elements related to formatting
+            (only valuable if output_format is set to XML).
+        deduplicate: Remove duplicate segments and documents.
+        date_extraction_params: Provide extraction parameters to htmldate as dict().
+        with_metadata: Only keep documents featuring all essential metadata
+            (date, title, url).
+        max_tree_size: Discard documents with too many elements.
+        url_blacklist: Provide a blacklist of URLs as set() to filter out documents.
+        config: Directly provide a configparser configuration.
+
+    Returns:
+        A Python dict() containing all the extracted information or None.
+
+    Raises:
+        ValueError: Extraction problem.
+    """
+    # init
     if url_blacklist is None:
         url_blacklist = set()
+
+    # load data
     try:
-        # load data
         tree = load_html(filecontent)
         if tree is None:
             raise ValueError
+
+        # HTML lang check
+        if target_language is not None and check_html_lang(tree, target_language) is False:
+            raise ValueError
+
         # backup (or not) for further processing
         if no_fallback is False:
             backup_tree = deepcopy(tree)
@@ -604,20 +672,20 @@ def bare_extraction(filecontent, url=None, no_fallback=False,
 
         # comments first, then remove
         if include_comments is True:
-            commentsbody, temp_comments, len_comments, cleaned_tree = extract_comments(cleaned_tree, deduplicate)
+            commentsbody, temp_comments, len_comments, cleaned_tree = extract_comments(cleaned_tree, deduplicate, config)
         else:
             commentsbody, temp_comments, len_comments = None, '', 0
 
         # extract content
-        postbody, temp_text, len_text, sure_thing = extract_content(cleaned_tree, include_tables, deduplicate)
+        postbody, temp_text, len_text, sure_thing = extract_content(cleaned_tree, include_tables, deduplicate, config)
 
         # compare if necessary
         if no_fallback is False:
             #if sure_thing is False:
-            postbody, temp_text, len_text = compare_extraction(tree, backup_tree, url, postbody, temp_text, len_text, target_language, include_formatting)
+            postbody, temp_text, len_text = compare_extraction(tree, backup_tree, url, postbody, temp_text, len_text, target_language, include_formatting, config)
         else:
             # rescue: try to use original/dirty tree
-            if sure_thing is False and len_text < MIN_EXTRACTED_SIZE:
+            if sure_thing is False and len_text < config.getint('DEFAULT', 'MIN_EXTRACTED_SIZE'):
                 postbody, temp_text, len_text = baseline(filecontent)
                 #tree = load_html(filecontent)
                 #tree = convert_tags(tree)
@@ -633,14 +701,15 @@ def bare_extraction(filecontent, url=None, no_fallback=False,
                     LOGGER.error('output tree too long: %s, discarding file', len(postbody))
                     raise ValueError
         # size checks
-        if len_comments < MIN_EXTRACTED_COMM_SIZE:
+        if len_comments < config.getint('DEFAULT', 'MIN_EXTRACTED_COMM_SIZE'):
             LOGGER.info('not enough comments %s', url)
-        if len_text < MIN_OUTPUT_SIZE and len_comments < MIN_OUTPUT_COMM_SIZE:
+        if len_text < config.getint('DEFAULT', 'MIN_OUTPUT_SIZE') and \
+           len_comments < config.getint('DEFAULT', 'MIN_OUTPUT_COMM_SIZE'):
             LOGGER.info('text and comments not long enough: %s %s', len_text, len_comments)
             raise ValueError
 
         # check duplicates at body level
-        if deduplicate is True and duplicate_test(postbody) is True:
+        if deduplicate is True and duplicate_test(postbody, config) is True:
             raise ValueError
 
         # sanity check on language
@@ -667,9 +736,44 @@ def extract(filecontent, url=None, record_id=None, no_fallback=False,
             csv_output=False, json_output=False, xml_output=False, tei_output=False,
             tei_validation=False, target_language=None,
             include_tables=True, include_images=False, include_formatting=False, deduplicate=False,
-            date_extraction_params=None, with_metadata=False, max_tree_size=None, url_blacklist=None):
-    '''Function exposed by the package:
-       wrapper for text extraction and conversion to chosen output format'''
+            date_extraction_params=None, with_metadata=False, max_tree_size=None, url_blacklist=None,
+            settingsfile=None, config=DEFAULT_CONFIG):
+    """Main function exposed by the package:
+       Wrapper for text extraction and conversion to chosen output format.
+
+    Args:
+        filecontent: HTML code as string.
+        url: URL of the webpage.
+        record_id: Add an ID to the metadata.
+        no_fallback: Skip the backup extraction with readability-lxml and justext.
+        include_comments: Extract comments along with the main text.
+        output_format: Define an output format:
+            'txt', 'csv', 'json', 'xml', or 'xmltei'.
+        csv_output: Set output format to CSV (alternative to output_format).
+        json_output: Set output format to JSON (alternative to output_format).
+        xml_output: Set output format to XML (alternative to output_format).
+        tei_output: Set output format to XML-TEI (alternative to output_format).
+        tei_validation: Validate the XML-TEI output with respect to the TEI standard.
+        target_language: Define a language to discard invalid documents (ISO 639-1 format).
+        include_tables: Take into account information within the HTML <table> element.
+        include_images: Take images into account.
+        include_formatting: Keep structural elements related to formatting
+            (only valuable if output_format is set to XML).
+        deduplicate: Remove duplicate segments and documents.
+        date_extraction_params: Provide extraction parameters to htmldate as dict().
+        with_metadata: Only keep documents featuring all essential metadata
+            (date, title, url).
+        max_tree_size: Discard documents with too many elements.
+        url_blacklist: Provide a blacklist of URLs as set() to filter out documents.
+        settingsfile: Use a configuration file to override the standard settings.
+        config: Directly provide a configparser configuration.
+
+    Returns:
+        A string in the desired format or None.
+
+    """
+    # configuration init
+    config = use_config(settingsfile, config)
     # metadata mapping for compatibility
     output_format = map_format(output_format, csv_output, json_output, xml_output, tei_output)
     if url_blacklist is None:
@@ -681,7 +785,7 @@ def extract(filecontent, url=None, record_id=None, no_fallback=False,
         target_language=target_language, include_tables=include_tables, include_images=include_images,
         include_formatting=include_formatting, deduplicate=deduplicate,
         date_extraction_params=date_extraction_params, with_metadata=with_metadata,
-        max_tree_size=max_tree_size, url_blacklist=url_blacklist
+        max_tree_size=max_tree_size, url_blacklist=url_blacklist, config=config,
         )
     if docmeta is None:
         return None
