@@ -16,7 +16,7 @@ except ImportError:
     LANGID_FLAG = False
 
 from .lru import LRUCache
-from .settings import LRU_SIZE, MAX_REPETITIONS, MIN_DUPLCHECK_SIZE
+from .settings import LRU_SIZE
 from .utils import trim  #, remove_control_characters
 
 
@@ -24,6 +24,8 @@ LOGGER = logging.getLogger(__name__)
 
 LRU_TEST = LRUCache(maxsize=LRU_SIZE)
 
+RE_HTML_LANG = re.compile(r'([a-z]{2})', re.I)
+
 RE_FILTER = re.compile(r'\W*(Drucken|E-?Mail|Facebook|Flipboard|Google|Instagram|Linkedin|Mail|PDF|Pinterest|Pocket|Print|Reddit|Twitter|Whatsapp|Xing)$', flags=re.IGNORECASE)
 # COMMENTS_BLACKLIST = ('( Abmelden / Ändern )') # Fill in your details below|Trage deine Daten unten|Kommentar verfassen|Bitte logge dich|Hinterlasse einen Kommentar| to %s| mit %s)
 
@@ -40,20 +42,34 @@ def put_in_cache(teststring):
         LRU_TEST.put(teststring, 1)
 
 
-def duplicate_test(element):
+def duplicate_test(element, config):
     '''Check for duplicate text with LRU cache'''
     teststring = trim(' '.join(element.itertext()))
     # teststring = element.text
-    if len(teststring) > MIN_DUPLCHECK_SIZE:
+    if len(teststring) > config.getint('DEFAULT', 'MIN_DUPLCHECK_SIZE'):
         # retrieve value from cache
         cacheval = LRU_TEST.get(teststring)
-        if cacheval > MAX_REPETITIONS:  # non-existent key will return -1
+        if cacheval > config.getint('DEFAULT', 'MAX_REPETITIONS'):  # non-existent key will return -1
             LRU_TEST.put(teststring, cacheval + 1)
             return True
     put_in_cache(teststring)
     return False
 
 
+def check_html_lang(tree, target_language):
+    '''Check HTML meta-elements for language information and split
+       the result in case there are several languages'''
+    # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Language
+    for elem in tree.xpath('//html[@lang]'):
+        if target_language in RE_HTML_LANG.split(elem.get('lang')):
+            return True
+    for elem in tree.xpath('//meta[@http-equiv="content-language"]'):
+        if target_language in RE_HTML_LANG.split(elem.get('content')):
+            return True
+    LOGGER.warning('HTML lang detection failed')
+    return False
+
+
 def language_filter(temp_text, temp_comments, target_language, docmeta):
     '''Run external component (if installed) for language identification'''
     # sanity check on language
