@@ -9,47 +9,37 @@ Module bundling functions related to HTML and text processing.
 import gzip
 import logging
 import re
-import socket
 import sys
 
 from functools import lru_cache
+from random import randint
 
+# CChardet is faster and can be more accurate
 try:
-    # this module is faster
-    import cchardet
+    import cchardet as chardet
 except ImportError:
-    cchardet = None
+    import chardet
 
-import requests
-#import urllib3
+import urllib3
 
 from lxml import etree, html
 # from lxml.html.soupparser import fromstring as fromsoup
 
-from requests.adapters import HTTPAdapter
-from requests.packages.urllib3.util.retry import Retry
-
-from .settings import MAX_FILE_SIZE, MIN_FILE_SIZE, TIMEOUT, USER_AGENT
+from .settings import MAX_FILE_SIZE, MIN_FILE_SIZE, TIMEOUT, USER_AGENTS, USER_AGENTS_NUM
 
 
 LOGGER = logging.getLogger(__name__)
 
 # customize headers
-HEADERS = {
-    # 'Connection': 'close',  # another way to cover tracks
-    'User-Agent': USER_AGENT,  # your string here
-}
-RETRY_STRATEGY = Retry(
+RETRY_STRATEGY = urllib3.util.Retry(
     total=3,
+    redirect=2, # raise_on_redirect=False,
+    connect=0,
     backoff_factor=TIMEOUT*2,
     status_forcelist=[429, 500, 502, 503, 504],
 )
-ADAPTER = HTTPAdapter(max_retries=RETRY_STRATEGY)
-# urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
-SESSION = requests.Session()
-SESSION.mount("https://", ADAPTER)
-SESSION.mount("http://", ADAPTER)
-SESSION.headers.update(HEADERS)
+urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
+HTTP_POOL = urllib3.PoolManager(retries=RETRY_STRATEGY)
 
 # collect_ids=False, default_doctype=False, huge_tree=True,
 HTML_PARSER = html.HTMLParser(remove_comments=True, remove_pis=True, encoding='utf-8')
@@ -74,7 +64,7 @@ NOPRINT_TRANS_TABLE = {
 IMAGE_EXTENSION = re.compile(r'([^\s]+(\.(jpe?g|png|gif|bmp)))')
 
 # Regex for crude extraction of host/domain name
-HOSTINFO = re.compile(r'https?://[^/]+')
+HOSTINFO = re.compile(r'(https?://[^/]+)')
 
 
 def is_gz_file(contents):
@@ -98,76 +88,95 @@ def detect_encoding(bytesobject):
     # unicode-test
     if isutf8(bytesobject):
         return 'UTF-8'
-    # try one of the installed detectors
-    if cchardet is not None:
-        guess = cchardet.detect(bytesobject)
-        LOGGER.debug('guessed encoding: %s', guess['encoding'])
-        return guess['encoding']
+    # try one of the installed detectors on first part
+    guess = chardet.detect(bytesobject[:1999])
+    LOGGER.debug('guessed encoding: %s, confidence: %s', guess['encoding'], guess['confidence'])
     # fallback on full response
-    #if guess is None or guess['encoding'] is None: # or guess['confidence'] < 0.99:
-    #    guessed_encoding = chardet.detect(bytesobject)['encoding']
-    # return
-    return None
+    if guess is None or guess['confidence'] < 0.95:
+        guess = chardet.detect(bytesobject)
+        LOGGER.debug('second-guessed encoding: %s, confidence: %s', guess['encoding'], guess['confidence'])
+    return guess['encoding']
 
 
 def decode_response(response):
-    """Read the Requests object corresponding to the server response,
+    """Read the urllib3 object corresponding to the server response,
        check if it could be GZip and eventually decompress it, then
        try to guess its encoding and decode it to return a unicode string"""
-    resp_content, resp_text = response.content, response.text
-    if is_gz_file(response.content):
-        decompressed = gzip.decompress(response.content)
-        resp_content, resp_text = decompressed, \
-            str(decompressed, encoding='utf-8', errors='replace')
+    if isinstance(response, bytes):
+        resp_content = response
+    elif is_gz_file(response.data):
+        resp_content = gzip.decompress(response.data)
+    else:
+        resp_content = response.data
     guessed_encoding = detect_encoding(resp_content)
-    LOGGER.debug('response/guessed encoding: %s / %s', response.encoding, guessed_encoding)
+    LOGGER.debug('response encoding: %s', guessed_encoding)
     # process
+    htmltext = None
     if guessed_encoding is not None:
         try:
             htmltext = resp_content.decode(guessed_encoding)
         except UnicodeDecodeError:
-            LOGGER.warning('encoding error: %s / %s', response.encoding, guessed_encoding)
-            htmltext = resp_text
-    else:
-        htmltext = resp_text
+            LOGGER.warning('encoding error: %s', guessed_encoding)
+    # force decoding # ascii instead?
+    if htmltext is None:
+        htmltext = str(resp_content, encoding='utf-8', errors='replace')
     return htmltext
 
 
-def fetch_url(url):
-    """ Fetch page using requests/urllib3 and decode the response
+def determine_headers():
+    'Decide on user-agent string'
+    if USER_AGENTS_NUM == 1:
+        rnumber = 0
+    else:
+        rnumber = randint(0, USER_AGENTS_NUM - 1)
+    headers = {
+    'User-Agent': USER_AGENTS[rnumber],
+    }
+    return headers
+
+
+def fetch_url(url, decode=True):
+    """Fetches page using urllib3 and decodes the response.
+
     Args:
-        URL: URL of the page to fetch
+        url: URL of the page to fetch.
+        decode: Decode response instead of returning Urllib3 response object (boolean).
+
     Returns:
-        request object (headers + body).
-    Raises:
-        Nothing.
+        HTML code as string, or Urllib3 response object (headers + body), or empty string in case
+        the result is invalid, or None if there was a problem with the network.
+
     """
     # send
     try:
         # read by streaming chunks (stream=True, iter_content=xx)
         # so we can stop downloading as soon as MAX_FILE_SIZE is reached
-        response = SESSION.get(url, timeout=TIMEOUT, verify=False, allow_redirects=True, headers=HEADERS)
-    except (requests.exceptions.MissingSchema, requests.exceptions.InvalidURL):
-        LOGGER.error('malformed URL: %s', url)
-    except requests.exceptions.TooManyRedirects:
-        LOGGER.error('redirects: %s', url)
-    except requests.exceptions.SSLError as err:
-        LOGGER.error('SSL: %s %s', url, err)
-    except (socket.timeout, requests.exceptions.ConnectionError,
-            requests.exceptions.Timeout, socket.error, socket.gaierror) as err:
-        LOGGER.error('connection: %s %s', url, err)
-    #except Exception as err:
-    #    logging.error('unknown: %s %s', url, err) # sys.exc_info()[0]
+        response = HTTP_POOL.request('GET', url, headers=determine_headers(),
+                                     timeout=TIMEOUT)
+    except urllib3.exceptions.NewConnectionError as err:
+        LOGGER.error('connection refused: %s %s', url, err)
+        return ''  # raise error instead?
+    except urllib3.exceptions.MaxRetryError as err:
+        LOGGER.error('retries/redirects: %s %s', url, err)
+        return ''  # raise error instead?
+    except urllib3.exceptions.TimeoutError as err:
+        LOGGER.error('connection timeout: %s %s', url, err)
+    except Exception as err:
+        logging.error('unknown error: %s %s', url, err) # sys.exc_info()[0]
     else:
         # safety checks
-        if response.status_code != 200:
-            LOGGER.error('not a 200 response: %s for URL %s', response.status_code, url)
-        elif response.text is None or len(response.text) < MIN_FILE_SIZE:
+        if response.status != 200:
+            LOGGER.error('not a 200 response: %s for URL %s', response.status, url)
+        elif response.data is None or len(response.data) < MIN_FILE_SIZE:
             LOGGER.error('too small/incorrect for URL %s', url)
-        elif len(response.text) > MAX_FILE_SIZE:
-            LOGGER.error('too large: length %s for URL %s', len(response.text), url)
+            return ''  # raise error instead?
+        elif len(response.data) > MAX_FILE_SIZE:
+            LOGGER.error('too large: length %s for URL %s', len(response.data), url)
+            return ''  # raise error instead?
         else:
-            return decode_response(response)
+            if decode is True:
+                return decode_response(response.data)
+            return response
     return None
 
 
